import uuid
import datetime
import typing
from abc import ABC, abstractmethod

from sensory_cloud.config import Config
from sensory_cloud.services.crypto_service import CryptoService

import sensory_cloud.generated.common.common_pb2 as common_pb2
import sensory_cloud.generated.oauth.oauth_pb2_grpc as oauth_pb2_grpc
import sensory_cloud.generated.oauth.oauth_pb2 as oauth_pb2
import sensory_cloud.generated.v1.management.device_pb2_grpc as device_pb2_grpc
import sensory_cloud.generated.v1.management.device_pb2 as device_pb2


class OauthClient:
    """
    Class that holds OAuth client id and secret
    """

    def __init__(
        self,
        client_id: str,
        client_secret: str,
    ):
        """
        Constructor method for the OauthClient class

        Arguments:
            client_id: String containing the client id
            client_secret: String containing the client secret
        """

        self._client_id = client_id
        self._client_secret = client_secret

    @property
    def client_id(self) -> str:
        """
        Get method for the client id attribute

        Returns:
            String containing the client id
        """

        return self._client_id

    @property
    def client_secret(self) -> str:
        """
        Get method for the client secret attribute

        Returns:
            String containing the client secret
        """

        return self._client_secret


class OAuthToken:
    """
    Class that holds OAuth token and expiration
    """

    def __init__(self, token: str, expires: datetime.datetime):
        """
        Constructor method for the OAuthToken class

        Arguments:
            token: String containing the oauth token
            expires: datetime.datetime object containing the token's
                expiration time stamp
        """

        self._token = token
        self._expires = expires

    @property
    def token(self) -> str:
        """
        Get method that returns the oauth token attribute

        Returns:
            String containing the oauth token
        """

        return self._token

    @property
    def expires(self) -> datetime.datetime:
        """
        Get method that returns the expiration date attribute

        Returns:
            A datetime.datetime object containing the token's
                expiration time stamp
        """

        return self._expires


class IOauthService(ABC):
    """
    Abstract class that manages OAuth interactions with Sensory Cloud
    """

    @abstractmethod
    def generate_credentials(self) -> OauthClient:
        """Method that generates a client id and a client secret"""

    @abstractmethod
    def get_token(self) -> OAuthToken:
        """Method that gets a token for the provided credentials"""

    @abstractmethod
    def register(
        self, device_id: str, device_name: str, credential: str
    ) -> device_pb2.DeviceResponse:
        """
        Method that registers credentials provided by the attached SecureCredentialStore to Sensory Cloud.
        This should only be called once per unique credential pair. An error will be thrown if registration fails.
        """


class ISecureCredentialStore(ABC):
    @abstractmethod
    def client_id(self):
        """Method that gets the client id"""

    @abstractmethod
    def client_secret(self):
        """Method that gets the client secret"""


class OauthService(IOauthService):
    """
    Class that manages OAuth interactions with Sensory Cloud
    """

    def __init__(self, config: Config, secure_credential_store: ISecureCredentialStore):
        """
        Constructor method for OauthService

        Arguments:
            config: Config object containing the relevant grpc connection information
            secure_credential_store: ISecureCredentialStore that stores the client id
                and client secret
        """

        self._config: Config = config
        self._oauth_client: oauth_pb2_grpc.OauthServiceStub = (
            oauth_pb2_grpc.OauthServiceStub(channel=config.channel)
        )
        self._device_client: device_pb2_grpc.DeviceServiceStub = (
            device_pb2_grpc.DeviceServiceStub(channel=config.channel)
        )
        self._secure_credential_store: ISecureCredentialStore = secure_credential_store

    @staticmethod
    def generate_credentials() -> OauthClient:
        """
        Can be called to generate secure and guaranteed unique credentials.
        Should be used the first time the SDK registers and OAuth token with the cloud.

        Returns:
            An OauthClient
        """

        client: OauthClient = OauthClient(
            client_id=str(uuid.uuid4()),
            client_secret=CryptoService().get_secure_random_string(length=24),
        )
        return client

    def get_who_am_i(self) -> device_pb2.DeviceResponse:
        """
        Get information about the current registered device as inferred by the OAuth credentials supplied by the credential manager.
        A new token is request every time this call is made, so use sparingly.

        Returns:
            Information about the current device
        """

        oauth_token: OAuthToken = self.get_token()
        metadata: typing.Tuple[typing.Tuple] = (
            ("authorization", f"Bearer {oauth_token.token}"),
        )

        return self._device_client.GetWhoAmI(
            request=device_pb2.DeviceGetWhoAmIRequest(), metadata=metadata
        )

    def get_token(self) -> OAuthToken:
        """
        Obtains an Oauth JWT from Sensory Cloud

        Returns:
            An OAuth JWT and expiration
        """

        client_id: str = self._secure_credential_store.client_id
        if client_id in [None, ""]:
            raise ValueError(
                "null client_id was returned from the secure credential store"
            )

        client_secret: str = self._secure_credential_store.client_secret
        if client_secret in [None, ""]:
            raise ValueError(
                "null client_secret was returned from the secure credential store"
            )

        now: datetime.datetime = datetime.datetime.utcnow()
        request: oauth_pb2.TokenRequest = oauth_pb2.TokenRequest(
            clientId=client_id, secret=client_secret
        )
        response: common_pb2.TokenResponse = self._oauth_client.GetToken(request)

        return OAuthToken(
            token=response.accessToken,
            expires=now + datetime.timedelta(response.expiresIn),
        )

    def register(
        self, device_id: str, device_name: str, credential: str
    ) -> device_pb2.DeviceResponse:
        """
        Register credentials provided by the attached SecureCredentialStore to Sensory Cloud.
        This function should only be called once per unique credential pair. An error will be
        thrown if registration fails.

        Arguments:
            device_id: String containing the uuid device id
            device_name: The friendly name of the registering device
            credential: The credential configured on the Sensory Cloud server

        Returns:
            A DeviceResponse indicating if the device was successfully registered
        """

        client_id: str = self._secure_credential_store.client_id
        if client_id in [None, ""]:
            raise ValueError(
                "null client_id was returned from the secure credential store"
            )

        client_secret: str = self._secure_credential_store.client_secret
        if client_secret in [None, ""]:
            raise ValueError(
                "null client_secret was returned from the secure credential store"
            )

        client: common_pb2.GenericClient = common_pb2.GenericClient(
            clientId=client_id, secret=client_secret
        )
        request: device_pb2.EnrollDeviceRequest = device_pb2.EnrollDeviceRequest(
            name=device_name,
            deviceId=device_id,
            tenantId=self._config.tenant_id,
            client=client,
            credential=credential,
        )
        device_response: device_pb2.DeviceResponse = self._device_client.EnrollDevice(
            request
        )

        return device_response

    def renew_device_credential(
        self, device_id: str, credential: str
    ) -> device_pb2.DeviceResponse:
        """
        Renew the credential associated with the given device

        Arguments:
            device_id: String containing the uuid device id
            credential: The credential configured on the Sensory Cloud server

        Returns:
            A DeviceResponse indicating if the device credential was changed
        """

        client_id: str = self._secure_credential_store.client_id
        if client_id in [None, ""]:
            raise ValueError(
                "null client_id was returned from the secure credential store"
            )

        request: device_pb2.RenewDeviceCredentialRequest = (
            device_pb2.RenewDeviceCredentialRequest(
                deviceId=device_id,
                clientId=client_id,
                tenantId=self._config.tenant_id,
                credential=credential,
            )
        )

        device_response: device_pb2.DeviceResponse = (
            self._device_client.RenewDeviceCredential(request=request)
        )

        return device_response


class GenericCredentialStore(ISecureCredentialStore):
    def __init__(self, client_id: str, client_secret: str):
        self._client_id = client_id
        self._client_secret = client_secret

    @property
    def client_id(self):
        return self._client_id

    @property
    def client_secret(self):
        return self._client_secret



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/management/device.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from sensory_cloud.generated.common import common_pb2 as common_dot_common__pb2
from sensory_cloud.generated.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='v1/management/device.proto',
  package='sensory.api.v1.management',
  syntax='proto3',
  serialized_options=b'\n!ai.sensorycloud.api.v1.managementB!SensoryApiV1ManagementDeviceProtoP\001Z?gitlab.com/sensory-cloud/server/titan.git/pkg/api/v1/management',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1av1/management/device.proto\x12\x19sensory.api.v1.management\x1a\x13\x63ommon/common.proto\x1a\x17validate/validate.proto\"\xb8\x01\n\x13\x45nrollDeviceRequest\x12\x17\n\x04name\x18\x01 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12\x1b\n\x08\x64\x65viceId\x18\x02 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12\x1a\n\x08tenantId\x18\x03 \x01(\tB\x08\xfa\x42\x05r\x03\xb0\x01\x01\x12;\n\x06\x63lient\x18\x04 \x01(\x0b\x32!.sensory.api.common.GenericClientB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x12\n\ncredential\x18\x05 \x01(\t\"\x93\x01\n\x1cRenewDeviceCredentialRequest\x12\x1b\n\x08\x64\x65viceId\x18\x01 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12\x1a\n\x08\x63lientId\x18\x02 \x01(\tB\x08\xfa\x42\x05r\x03\xb0\x01\x01\x12\x1a\n\x08tenantId\x18\x03 \x01(\tB\x08\xfa\x42\x05r\x03\xb0\x01\x01\x12\x1e\n\ncredential\x18\x04 \x01(\tB\n\xfa\x42\x07r\x05\x10\x01\x18\xff\x01\"\x18\n\x16\x44\x65viceGetWhoAmIRequest\",\n\rDeviceRequest\x12\x1b\n\x08\x64\x65viceId\x18\x01 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\"y\n\x11GetDevicesRequest\x12\x10\n\x08tenantId\x18\x01 \x01(\t\x12\x39\n\npagination\x18\x02 \x01(\x0b\x32%.sensory.api.common.PaginationOptions\x12\x17\n\x06userId\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x18\x7f\"K\n\x13UpdateDeviceRequest\x12\x1b\n\x08\x64\x65viceId\x18\x01 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12\x17\n\x04name\x18\x02 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\"0\n\x0e\x44\x65viceResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x65viceId\x18\x02 \x01(\t\"F\n\x11GetDeviceResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x65viceId\x18\x02 \x01(\t\x12\x11\n\tuserCount\x18\x03 \x01(\x03\"\x8c\x01\n\x12\x44\x65viceListResponse\x12:\n\x07\x64\x65vices\x18\x01 \x03(\x0b\x32).sensory.api.v1.management.DeviceResponse\x12:\n\npagination\x18\x02 \x01(\x0b\x32&.sensory.api.common.PaginationResponse2\x90\x06\n\rDeviceService\x12k\n\x0c\x45nrollDevice\x12..sensory.api.v1.management.EnrollDeviceRequest\x1a).sensory.api.v1.management.DeviceResponse\"\x00\x12}\n\x15RenewDeviceCredential\x12\x37.sensory.api.v1.management.RenewDeviceCredentialRequest\x1a).sensory.api.v1.management.DeviceResponse\"\x00\x12k\n\tGetWhoAmI\x12\x31.sensory.api.v1.management.DeviceGetWhoAmIRequest\x1a).sensory.api.v1.management.DeviceResponse\"\x00\x12\x65\n\tGetDevice\x12(.sensory.api.v1.management.DeviceRequest\x1a,.sensory.api.v1.management.GetDeviceResponse\"\x00\x12k\n\nGetDevices\x12,.sensory.api.v1.management.GetDevicesRequest\x1a-.sensory.api.v1.management.DeviceListResponse\"\x00\x12k\n\x0cUpdateDevice\x12..sensory.api.v1.management.UpdateDeviceRequest\x1a).sensory.api.v1.management.DeviceResponse\"\x00\x12\x65\n\x0c\x44\x65leteDevice\x12(.sensory.api.v1.management.DeviceRequest\x1a).sensory.api.v1.management.DeviceResponse\"\x00\x42\x89\x01\n!ai.sensorycloud.api.v1.managementB!SensoryApiV1ManagementDeviceProtoP\x01Z?gitlab.com/sensory-cloud/server/titan.git/pkg/api/v1/managementb\x06proto3'
  ,
  dependencies=[common_dot_common__pb2.DESCRIPTOR,validate_dot_validate__pb2.DESCRIPTOR,])




_ENROLLDEVICEREQUEST = _descriptor.Descriptor(
  name='EnrollDeviceRequest',
  full_name='sensory.api.v1.management.EnrollDeviceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='sensory.api.v1.management.EnrollDeviceRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceId', full_name='sensory.api.v1.management.EnrollDeviceRequest.deviceId', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tenantId', full_name='sensory.api.v1.management.EnrollDeviceRequest.tenantId', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005r\003\260\001\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client', full_name='sensory.api.v1.management.EnrollDeviceRequest.client', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005\212\001\002\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='credential', full_name='sensory.api.v1.management.EnrollDeviceRequest.credential', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=104,
  serialized_end=288,
)


_RENEWDEVICECREDENTIALREQUEST = _descriptor.Descriptor(
  name='RenewDeviceCredentialRequest',
  full_name='sensory.api.v1.management.RenewDeviceCredentialRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceId', full_name='sensory.api.v1.management.RenewDeviceCredentialRequest.deviceId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clientId', full_name='sensory.api.v1.management.RenewDeviceCredentialRequest.clientId', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005r\003\260\001\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tenantId', full_name='sensory.api.v1.management.RenewDeviceCredentialRequest.tenantId', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005r\003\260\001\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='credential', full_name='sensory.api.v1.management.RenewDeviceCredentialRequest.credential', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\007r\005\020\001\030\377\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=291,
  serialized_end=438,
)


_DEVICEGETWHOAMIREQUEST = _descriptor.Descriptor(
  name='DeviceGetWhoAmIRequest',
  full_name='sensory.api.v1.management.DeviceGetWhoAmIRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=440,
  serialized_end=464,
)


_DEVICEREQUEST = _descriptor.Descriptor(
  name='DeviceRequest',
  full_name='sensory.api.v1.management.DeviceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceId', full_name='sensory.api.v1.management.DeviceRequest.deviceId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=466,
  serialized_end=510,
)


_GETDEVICESREQUEST = _descriptor.Descriptor(
  name='GetDevicesRequest',
  full_name='sensory.api.v1.management.GetDevicesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tenantId', full_name='sensory.api.v1.management.GetDevicesRequest.tenantId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pagination', full_name='sensory.api.v1.management.GetDevicesRequest.pagination', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='userId', full_name='sensory.api.v1.management.GetDevicesRequest.userId', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\004r\002\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=512,
  serialized_end=633,
)


_UPDATEDEVICEREQUEST = _descriptor.Descriptor(
  name='UpdateDeviceRequest',
  full_name='sensory.api.v1.management.UpdateDeviceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceId', full_name='sensory.api.v1.management.UpdateDeviceRequest.deviceId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sensory.api.v1.management.UpdateDeviceRequest.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=635,
  serialized_end=710,
)


_DEVICERESPONSE = _descriptor.Descriptor(
  name='DeviceResponse',
  full_name='sensory.api.v1.management.DeviceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='sensory.api.v1.management.DeviceResponse.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceId', full_name='sensory.api.v1.management.DeviceResponse.deviceId', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=712,
  serialized_end=760,
)


_GETDEVICERESPONSE = _descriptor.Descriptor(
  name='GetDeviceResponse',
  full_name='sensory.api.v1.management.GetDeviceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='sensory.api.v1.management.GetDeviceResponse.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceId', full_name='sensory.api.v1.management.GetDeviceResponse.deviceId', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='userCount', full_name='sensory.api.v1.management.GetDeviceResponse.userCount', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=762,
  serialized_end=832,
)


_DEVICELISTRESPONSE = _descriptor.Descriptor(
  name='DeviceListResponse',
  full_name='sensory.api.v1.management.DeviceListResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='devices', full_name='sensory.api.v1.management.DeviceListResponse.devices', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pagination', full_name='sensory.api.v1.management.DeviceListResponse.pagination', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=835,
  serialized_end=975,
)

_ENROLLDEVICEREQUEST.fields_by_name['client'].message_type = common_dot_common__pb2._GENERICCLIENT
_GETDEVICESREQUEST.fields_by_name['pagination'].message_type = common_dot_common__pb2._PAGINATIONOPTIONS
_DEVICELISTRESPONSE.fields_by_name['devices'].message_type = _DEVICERESPONSE
_DEVICELISTRESPONSE.fields_by_name['pagination'].message_type = common_dot_common__pb2._PAGINATIONRESPONSE
DESCRIPTOR.message_types_by_name['EnrollDeviceRequest'] = _ENROLLDEVICEREQUEST
DESCRIPTOR.message_types_by_name['RenewDeviceCredentialRequest'] = _RENEWDEVICECREDENTIALREQUEST
DESCRIPTOR.message_types_by_name['DeviceGetWhoAmIRequest'] = _DEVICEGETWHOAMIREQUEST
DESCRIPTOR.message_types_by_name['DeviceRequest'] = _DEVICEREQUEST
DESCRIPTOR.message_types_by_name['GetDevicesRequest'] = _GETDEVICESREQUEST
DESCRIPTOR.message_types_by_name['UpdateDeviceRequest'] = _UPDATEDEVICEREQUEST
DESCRIPTOR.message_types_by_name['DeviceResponse'] = _DEVICERESPONSE
DESCRIPTOR.message_types_by_name['GetDeviceResponse'] = _GETDEVICERESPONSE
DESCRIPTOR.message_types_by_name['DeviceListResponse'] = _DEVICELISTRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EnrollDeviceRequest = _reflection.GeneratedProtocolMessageType('EnrollDeviceRequest', (_message.Message,), {
  'DESCRIPTOR' : _ENROLLDEVICEREQUEST,
  '__module__' : 'v1.management.device_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.EnrollDeviceRequest)
  })
_sym_db.RegisterMessage(EnrollDeviceRequest)

RenewDeviceCredentialRequest = _reflection.GeneratedProtocolMessageType('RenewDeviceCredentialRequest', (_message.Message,), {
  'DESCRIPTOR' : _RENEWDEVICECREDENTIALREQUEST,
  '__module__' : 'v1.management.device_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.RenewDeviceCredentialRequest)
  })
_sym_db.RegisterMessage(RenewDeviceCredentialRequest)

DeviceGetWhoAmIRequest = _reflection.GeneratedProtocolMessageType('DeviceGetWhoAmIRequest', (_message.Message,), {
  'DESCRIPTOR' : _DEVICEGETWHOAMIREQUEST,
  '__module__' : 'v1.management.device_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.DeviceGetWhoAmIRequest)
  })
_sym_db.RegisterMessage(DeviceGetWhoAmIRequest)

DeviceRequest = _reflection.GeneratedProtocolMessageType('DeviceRequest', (_message.Message,), {
  'DESCRIPTOR' : _DEVICEREQUEST,
  '__module__' : 'v1.management.device_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.DeviceRequest)
  })
_sym_db.RegisterMessage(DeviceRequest)

GetDevicesRequest = _reflection.GeneratedProtocolMessageType('GetDevicesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDEVICESREQUEST,
  '__module__' : 'v1.management.device_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.GetDevicesRequest)
  })
_sym_db.RegisterMessage(GetDevicesRequest)

UpdateDeviceRequest = _reflection.GeneratedProtocolMessageType('UpdateDeviceRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDEVICEREQUEST,
  '__module__' : 'v1.management.device_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.UpdateDeviceRequest)
  })
_sym_db.RegisterMessage(UpdateDeviceRequest)

DeviceResponse = _reflection.GeneratedProtocolMessageType('DeviceResponse', (_message.Message,), {
  'DESCRIPTOR' : _DEVICERESPONSE,
  '__module__' : 'v1.management.device_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.DeviceResponse)
  })
_sym_db.RegisterMessage(DeviceResponse)

GetDeviceResponse = _reflection.GeneratedProtocolMessageType('GetDeviceResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETDEVICERESPONSE,
  '__module__' : 'v1.management.device_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.GetDeviceResponse)
  })
_sym_db.RegisterMessage(GetDeviceResponse)

DeviceListResponse = _reflection.GeneratedProtocolMessageType('DeviceListResponse', (_message.Message,), {
  'DESCRIPTOR' : _DEVICELISTRESPONSE,
  '__module__' : 'v1.management.device_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.management.DeviceListResponse)
  })
_sym_db.RegisterMessage(DeviceListResponse)


DESCRIPTOR._options = None
_ENROLLDEVICEREQUEST.fields_by_name['name']._options = None
_ENROLLDEVICEREQUEST.fields_by_name['deviceId']._options = None
_ENROLLDEVICEREQUEST.fields_by_name['tenantId']._options = None
_ENROLLDEVICEREQUEST.fields_by_name['client']._options = None
_RENEWDEVICECREDENTIALREQUEST.fields_by_name['deviceId']._options = None
_RENEWDEVICECREDENTIALREQUEST.fields_by_name['clientId']._options = None
_RENEWDEVICECREDENTIALREQUEST.fields_by_name['tenantId']._options = None
_RENEWDEVICECREDENTIALREQUEST.fields_by_name['credential']._options = None
_DEVICEREQUEST.fields_by_name['deviceId']._options = None
_GETDEVICESREQUEST.fields_by_name['userId']._options = None
_UPDATEDEVICEREQUEST.fields_by_name['deviceId']._options = None
_UPDATEDEVICEREQUEST.fields_by_name['name']._options = None

_DEVICESERVICE = _descriptor.ServiceDescriptor(
  name='DeviceService',
  full_name='sensory.api.v1.management.DeviceService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=978,
  serialized_end=1762,
  methods=[
  _descriptor.MethodDescriptor(
    name='EnrollDevice',
    full_name='sensory.api.v1.management.DeviceService.EnrollDevice',
    index=0,
    containing_service=None,
    input_type=_ENROLLDEVICEREQUEST,
    output_type=_DEVICERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='RenewDeviceCredential',
    full_name='sensory.api.v1.management.DeviceService.RenewDeviceCredential',
    index=1,
    containing_service=None,
    input_type=_RENEWDEVICECREDENTIALREQUEST,
    output_type=_DEVICERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetWhoAmI',
    full_name='sensory.api.v1.management.DeviceService.GetWhoAmI',
    index=2,
    containing_service=None,
    input_type=_DEVICEGETWHOAMIREQUEST,
    output_type=_DEVICERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDevice',
    full_name='sensory.api.v1.management.DeviceService.GetDevice',
    index=3,
    containing_service=None,
    input_type=_DEVICEREQUEST,
    output_type=_GETDEVICERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDevices',
    full_name='sensory.api.v1.management.DeviceService.GetDevices',
    index=4,
    containing_service=None,
    input_type=_GETDEVICESREQUEST,
    output_type=_DEVICELISTRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateDevice',
    full_name='sensory.api.v1.management.DeviceService.UpdateDevice',
    index=5,
    containing_service=None,
    input_type=_UPDATEDEVICEREQUEST,
    output_type=_DEVICERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteDevice',
    full_name='sensory.api.v1.management.DeviceService.DeleteDevice',
    index=6,
    containing_service=None,
    input_type=_DEVICEREQUEST,
    output_type=_DEVICERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_DEVICESERVICE)

DESCRIPTOR.services_by_name['DeviceService'] = _DEVICESERVICE

# @@protoc_insertion_point(module_scope)



import datetime
import threading
import typing
from abc import ABC, abstractmethod

from sensory_cloud.services.oauth_service import IOauthService, OAuthToken


Metadata = typing.Tuple[typing.Tuple]


class ITokenManager(ABC):
    """
    Abstract class to handle the management of OAuth Tokens
    """

    @abstractmethod
    def get_token(self) -> str:
        """
        Method that gets a valid oath token

        Returns:
            A JWT as a string
        """

    @abstractmethod
    def get_authorization_metadata(self) -> Metadata:
        """
        Method that gets a token wrapped in grpc metadata

        Returns:
            A JWT wrapped in grpc metadata
        """


class TokenManager(ITokenManager):
    """
    Manages the rotation and injection of OAuth JWTs into grpc requests
    """

    _expires_buffer_seconds: int = 60 ** 2
    _token_mutex: threading.Lock = threading.Lock()
    _token: str = None
    _expires: datetime.datetime = None

    def __init__(self, oauth_service: IOauthService):
        self.oauth_service: IOauthService = oauth_service

    def get_token(self) -> str:

        self._token_mutex.acquire()

        if (
            self._token not in [None, ""]
            and self._expires is not None
            and datetime.datetime.utcnow() < self._expires
        ):
            self._token_mutex.release()
            return self._token

        try:
            oath_token = self.oauth_service.get_token()
            self._set_token(oath_token)
            self._token_mutex.release()
            return oath_token.token
        except Exception as e:
            self._token_mutex.release()
            raise (e)

    def get_authorization_metadata(self) -> Metadata:

        token: str = self.get_token()
        return (("authorization", f"Bearer {token}"),)

    def _set_token(self, oath_token: OAuthToken) -> None:
        """
        Private method that sets the _token and _expires arguments if the
        token has expired
        """

        self._token = oath_token.token
        self._expires = oath_token.expires - datetime.timedelta(
            seconds=self._expires_buffer_seconds
        )



import typing
from enum import Enum

from sensory_cloud.config import Config
from sensory_cloud.token_manager import ITokenManager, Metadata

import sensory_cloud.generated.v1.assistant.assistant_pb2 as assistant_pb2
import sensory_cloud.generated.v1.assistant.assistant_pb2_grpc as assistant_pb2_grpc


class RequestIterator:
    """
    The RequestIterator class facilitates the request streams that are sent to the
    grpc server.  There are six possible audio request types and and five request configurations
    that are given by the AudioRequest and RequestConfig enums respectively.  The first
    request sent must be a configuration request and all subsequent requests contain the audio
    content being streamed and any post processing actions if relevant.
    """

    _first_request: bool = True

    def __init__(
        self,
        request_config: assistant_pb2.AssistantMessageConfig,
        assistant_stream_iterator: typing.Iterable[typing.Union[bytes, str]],
    ):
        """
        Constructor method for the RequestIterator class

        Arguments:
            audio_request: AudioRequest enum denoting which type of request is being sent
            request_config: RequestConfig enum containing the initial request configuration
            audio_stream_iterator: Iterator containing audio bytes
        """

        self._request_config = request_config
        self._assistant_stream_iterator = assistant_stream_iterator

    def __iter__(self):
        return self

    def __next__(self) -> assistant_pb2.AssistantMessageRequest:
        if self._first_request:
            self._first_request = False
            return assistant_pb2.AssistantMessageRequest(config=self._request_config)
        else:
            return assistant_pb2.AssistantMessageRequest(
                message=next(self._assistant_stream_iterator)
            )


class AssistantService:
    """
    Class that handles communication with the sensory cloud assistant service
    """

    def __init__(self, config: Config, token_manager: ITokenManager):
        self._config: Config = config
        self._token_manager: ITokenManager = token_manager
        self._assistant_service_client: assistant_pb2_grpc.AssistantServiceStub = (
            assistant_pb2_grpc.AssistantServiceStub(config.channel)
        )

    def process_message(
        self,
        user_id: str,
        device_id: str,
        model_name: str,
        include_audio_response: bool,
        message_iterator: typing.Iterable[assistant_pb2.AssistantMessage],
    ) -> typing.Iterable[assistant_pb2.AssistantMessageResponse]:
        """
        Method that sends messages to the sensory cloud assistant to be processed

        Arguments:
            user_id: String containing the user id
            device_id: String containing the device id
            model_name: String containing the model name
            include_audio_response: Boolean indicating whether or not audio should be included in the response
            message_iterator: Iterator of AssistantMessage objects

        Returns:
            An iterator of AssistantMessageResponse objects
        """

        config: assistant_pb2.AssistantMessageConfig = (
            assistant_pb2.AssistantMessageConfig(
                userId=user_id,
                deviceId=device_id,
                modelName=model_name,
                includeAudioResponse=include_audio_response,
            )
        )

        request_iterator: RequestIterator = RequestIterator(
            request_config=config, assistant_stream_iterator=message_iterator
        )

        metadata: Metadata = self._token_manager.get_authorization_metadata()

        assistant_stream: typing.Iterable[
            assistant_pb2.AssistantMessageResponse
        ] = self._assistant_service_client.ProcessMessage(
            request_iterator=request_iterator, metadata=metadata
        )

        return assistant_stream



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from sensory_cloud.generated.v1.audio import audio_pb2 as v1_dot_audio_dot_audio__pb2


class AudioModelsStub(object):
    """Handles the retrieval and management of audio models
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetModels = channel.unary_unary(
                '/sensory.api.v1.audio.AudioModels/GetModels',
                request_serializer=v1_dot_audio_dot_audio__pb2.GetModelsRequest.SerializeToString,
                response_deserializer=v1_dot_audio_dot_audio__pb2.GetModelsResponse.FromString,
                )


class AudioModelsServicer(object):
    """Handles the retrieval and management of audio models
    """

    def GetModels(self, request, context):
        """Get available models for enrollment and authentication
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AudioModelsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetModels': grpc.unary_unary_rpc_method_handler(
                    servicer.GetModels,
                    request_deserializer=v1_dot_audio_dot_audio__pb2.GetModelsRequest.FromString,
                    response_serializer=v1_dot_audio_dot_audio__pb2.GetModelsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'sensory.api.v1.audio.AudioModels', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AudioModels(object):
    """Handles the retrieval and management of audio models
    """

    @staticmethod
    def GetModels(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.v1.audio.AudioModels/GetModels',
            v1_dot_audio_dot_audio__pb2.GetModelsRequest.SerializeToString,
            v1_dot_audio_dot_audio__pb2.GetModelsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class AudioBiometricsStub(object):
    """Handles all audio-related biometrics
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateEnrollment = channel.stream_stream(
                '/sensory.api.v1.audio.AudioBiometrics/CreateEnrollment',
                request_serializer=v1_dot_audio_dot_audio__pb2.CreateEnrollmentRequest.SerializeToString,
                response_deserializer=v1_dot_audio_dot_audio__pb2.CreateEnrollmentResponse.FromString,
                )
        self.Authenticate = channel.stream_stream(
                '/sensory.api.v1.audio.AudioBiometrics/Authenticate',
                request_serializer=v1_dot_audio_dot_audio__pb2.AuthenticateRequest.SerializeToString,
                response_deserializer=v1_dot_audio_dot_audio__pb2.AuthenticateResponse.FromString,
                )


class AudioBiometricsServicer(object):
    """Handles all audio-related biometrics
    """

    def CreateEnrollment(self, request_iterator, context):
        """Enrolls a user with a stream of audio. Streams a CreateEnrollmentResponse as the audio is processed.
        CreateEnrollment only supports biometric-enabled models
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Authenticate(self, request_iterator, context):
        """Authenticates a user with a stream of audio against an existing enrollment.
        Streams an AuthenticateResponse as the audio is processed.
        Authenticate only supports biometric-enabled models
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AudioBiometricsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateEnrollment': grpc.stream_stream_rpc_method_handler(
                    servicer.CreateEnrollment,
                    request_deserializer=v1_dot_audio_dot_audio__pb2.CreateEnrollmentRequest.FromString,
                    response_serializer=v1_dot_audio_dot_audio__pb2.CreateEnrollmentResponse.SerializeToString,
            ),
            'Authenticate': grpc.stream_stream_rpc_method_handler(
                    servicer.Authenticate,
                    request_deserializer=v1_dot_audio_dot_audio__pb2.AuthenticateRequest.FromString,
                    response_serializer=v1_dot_audio_dot_audio__pb2.AuthenticateResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'sensory.api.v1.audio.AudioBiometrics', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AudioBiometrics(object):
    """Handles all audio-related biometrics
    """

    @staticmethod
    def CreateEnrollment(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/sensory.api.v1.audio.AudioBiometrics/CreateEnrollment',
            v1_dot_audio_dot_audio__pb2.CreateEnrollmentRequest.SerializeToString,
            v1_dot_audio_dot_audio__pb2.CreateEnrollmentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Authenticate(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/sensory.api.v1.audio.AudioBiometrics/Authenticate',
            v1_dot_audio_dot_audio__pb2.AuthenticateRequest.SerializeToString,
            v1_dot_audio_dot_audio__pb2.AuthenticateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class AudioEventsStub(object):
    """Handles all audio event processing
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ValidateEvent = channel.stream_stream(
                '/sensory.api.v1.audio.AudioEvents/ValidateEvent',
                request_serializer=v1_dot_audio_dot_audio__pb2.ValidateEventRequest.SerializeToString,
                response_deserializer=v1_dot_audio_dot_audio__pb2.ValidateEventResponse.FromString,
                )
        self.CreateEnrolledEvent = channel.stream_stream(
                '/sensory.api.v1.audio.AudioEvents/CreateEnrolledEvent',
                request_serializer=v1_dot_audio_dot_audio__pb2.CreateEnrolledEventRequest.SerializeToString,
                response_deserializer=v1_dot_audio_dot_audio__pb2.CreateEnrollmentResponse.FromString,
                )
        self.ValidateEnrolledEvent = channel.stream_stream(
                '/sensory.api.v1.audio.AudioEvents/ValidateEnrolledEvent',
                request_serializer=v1_dot_audio_dot_audio__pb2.ValidateEnrolledEventRequest.SerializeToString,
                response_deserializer=v1_dot_audio_dot_audio__pb2.ValidateEnrolledEventResponse.FromString,
                )


class AudioEventsServicer(object):
    """Handles all audio event processing
    """

    def ValidateEvent(self, request_iterator, context):
        """Validates a phrase or sound with a stream of audio.
        Streams a ValidateEventResponse as the audio is processed.
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateEnrolledEvent(self, request_iterator, context):
        """Enrolls a sound or voice. Streams a CreateEnrollmentResponse as the audio is processed.
        CreateEnrollment supports all enrollable models
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ValidateEnrolledEvent(self, request_iterator, context):
        """Authenticates a sound or voice. Streams a ValidateEventResponse as the audio is processed.
        ValidateEnrolledEvent supports all enrollable models
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AudioEventsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ValidateEvent': grpc.stream_stream_rpc_method_handler(
                    servicer.ValidateEvent,
                    request_deserializer=v1_dot_audio_dot_audio__pb2.ValidateEventRequest.FromString,
                    response_serializer=v1_dot_audio_dot_audio__pb2.ValidateEventResponse.SerializeToString,
            ),
            'CreateEnrolledEvent': grpc.stream_stream_rpc_method_handler(
                    servicer.CreateEnrolledEvent,
                    request_deserializer=v1_dot_audio_dot_audio__pb2.CreateEnrolledEventRequest.FromString,
                    response_serializer=v1_dot_audio_dot_audio__pb2.CreateEnrollmentResponse.SerializeToString,
            ),
            'ValidateEnrolledEvent': grpc.stream_stream_rpc_method_handler(
                    servicer.ValidateEnrolledEvent,
                    request_deserializer=v1_dot_audio_dot_audio__pb2.ValidateEnrolledEventRequest.FromString,
                    response_serializer=v1_dot_audio_dot_audio__pb2.ValidateEnrolledEventResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'sensory.api.v1.audio.AudioEvents', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AudioEvents(object):
    """Handles all audio event processing
    """

    @staticmethod
    def ValidateEvent(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/sensory.api.v1.audio.AudioEvents/ValidateEvent',
            v1_dot_audio_dot_audio__pb2.ValidateEventRequest.SerializeToString,
            v1_dot_audio_dot_audio__pb2.ValidateEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateEnrolledEvent(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/sensory.api.v1.audio.AudioEvents/CreateEnrolledEvent',
            v1_dot_audio_dot_audio__pb2.CreateEnrolledEventRequest.SerializeToString,
            v1_dot_audio_dot_audio__pb2.CreateEnrollmentResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ValidateEnrolledEvent(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/sensory.api.v1.audio.AudioEvents/ValidateEnrolledEvent',
            v1_dot_audio_dot_audio__pb2.ValidateEnrolledEventRequest.SerializeToString,
            v1_dot_audio_dot_audio__pb2.ValidateEnrolledEventResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class AudioTranscriptionsStub(object):
    """Handles all audio transcriptions
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Transcribe = channel.stream_stream(
                '/sensory.api.v1.audio.AudioTranscriptions/Transcribe',
                request_serializer=v1_dot_audio_dot_audio__pb2.TranscribeRequest.SerializeToString,
                response_deserializer=v1_dot_audio_dot_audio__pb2.TranscribeResponse.FromString,
                )


class AudioTranscriptionsServicer(object):
    """Handles all audio transcriptions
    """

    def Transcribe(self, request_iterator, context):
        """Transcribes voice into text.
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AudioTranscriptionsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Transcribe': grpc.stream_stream_rpc_method_handler(
                    servicer.Transcribe,
                    request_deserializer=v1_dot_audio_dot_audio__pb2.TranscribeRequest.FromString,
                    response_serializer=v1_dot_audio_dot_audio__pb2.TranscribeResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'sensory.api.v1.audio.AudioTranscriptions', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AudioTranscriptions(object):
    """Handles all audio transcriptions
    """

    @staticmethod
    def Transcribe(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/sensory.api.v1.audio.AudioTranscriptions/Transcribe',
            v1_dot_audio_dot_audio__pb2.TranscribeRequest.SerializeToString,
            v1_dot_audio_dot_audio__pb2.TranscribeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class AudioSynthesisStub(object):
    """Handles synthesizing audio from text
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SynthesizeSpeech = channel.unary_stream(
                '/sensory.api.v1.audio.AudioSynthesis/SynthesizeSpeech',
                request_serializer=v1_dot_audio_dot_audio__pb2.SynthesizeSpeechRequest.SerializeToString,
                response_deserializer=v1_dot_audio_dot_audio__pb2.SynthesizeSpeechResponse.FromString,
                )


class AudioSynthesisServicer(object):
    """Handles synthesizing audio from text
    """

    def SynthesizeSpeech(self, request, context):
        """Synthesizes speech from text
        Authorization metadata is required {"authorization": "Bearer <TOKNE>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AudioSynthesisServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SynthesizeSpeech': grpc.unary_stream_rpc_method_handler(
                    servicer.SynthesizeSpeech,
                    request_deserializer=v1_dot_audio_dot_audio__pb2.SynthesizeSpeechRequest.FromString,
                    response_serializer=v1_dot_audio_dot_audio__pb2.SynthesizeSpeechResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'sensory.api.v1.audio.AudioSynthesis', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AudioSynthesis(object):
    """Handles synthesizing audio from text
    """

    @staticmethod
    def SynthesizeSpeech(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/sensory.api.v1.audio.AudioSynthesis/SynthesizeSpeech',
            v1_dot_audio_dot_audio__pb2.SynthesizeSpeechRequest.SerializeToString,
            v1_dot_audio_dot_audio__pb2.SynthesizeSpeechResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from sensory_cloud.generated.common import common_pb2 as common_dot_common__pb2
from sensory_cloud.generated.oauth import oauth_pb2 as oauth_dot_oauth__pb2


class OauthServiceStub(object):
    """Service for OAuth function
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetToken = channel.unary_unary(
                '/sensory.api.oauth.OauthService/GetToken',
                request_serializer=oauth_dot_oauth__pb2.TokenRequest.SerializeToString,
                response_deserializer=common_dot_common__pb2.TokenResponse.FromString,
                )
        self.SignToken = channel.unary_unary(
                '/sensory.api.oauth.OauthService/SignToken',
                request_serializer=oauth_dot_oauth__pb2.SignTokenRequest.SerializeToString,
                response_deserializer=common_dot_common__pb2.TokenResponse.FromString,
                )
        self.GetWhoAmI = channel.unary_unary(
                '/sensory.api.oauth.OauthService/GetWhoAmI',
                request_serializer=oauth_dot_oauth__pb2.WhoAmIRequest.SerializeToString,
                response_deserializer=oauth_dot_oauth__pb2.WhoAmIResponse.FromString,
                )
        self.GetPublicKey = channel.unary_unary(
                '/sensory.api.oauth.OauthService/GetPublicKey',
                request_serializer=oauth_dot_oauth__pb2.PublicKeyRequest.SerializeToString,
                response_deserializer=oauth_dot_oauth__pb2.PublicKeyResponse.FromString,
                )


class OauthServiceServicer(object):
    """Service for OAuth function
    """

    def GetToken(self, request, context):
        """Obtain an OAuth token for the given credentials
        Endpoint does not require an authorization token
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SignToken(self, request, context):
        """Sign and return an OAuth token. The passed authorization token must have the SignToken authority.
        Therefore, Devices are not allowed to make this request.
        Sign does not validate credentials, and therefore should be used in specific circumstances where credentials are not required.
        One common usecase for the Sign request is an Io server issuing a user-scoped token after a successful authentication.
        Only a limited subset of of scopes may be requested from the SignToken request.
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetWhoAmI(self, request, context):
        """Obtain information about oneself based on the passed OAuth token
        Authorization metadata is required {"authorization": "Bearer <TOKEN>"}
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPublicKey(self, request, context):
        """Retrieve a public key by ID. Public keys retrieved by this endpoint can be used
        to validate tokens signed by the Sensory cloud.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_OauthServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetToken': grpc.unary_unary_rpc_method_handler(
                    servicer.GetToken,
                    request_deserializer=oauth_dot_oauth__pb2.TokenRequest.FromString,
                    response_serializer=common_dot_common__pb2.TokenResponse.SerializeToString,
            ),
            'SignToken': grpc.unary_unary_rpc_method_handler(
                    servicer.SignToken,
                    request_deserializer=oauth_dot_oauth__pb2.SignTokenRequest.FromString,
                    response_serializer=common_dot_common__pb2.TokenResponse.SerializeToString,
            ),
            'GetWhoAmI': grpc.unary_unary_rpc_method_handler(
                    servicer.GetWhoAmI,
                    request_deserializer=oauth_dot_oauth__pb2.WhoAmIRequest.FromString,
                    response_serializer=oauth_dot_oauth__pb2.WhoAmIResponse.SerializeToString,
            ),
            'GetPublicKey': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPublicKey,
                    request_deserializer=oauth_dot_oauth__pb2.PublicKeyRequest.FromString,
                    response_serializer=oauth_dot_oauth__pb2.PublicKeyResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'sensory.api.oauth.OauthService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class OauthService(object):
    """Service for OAuth function
    """

    @staticmethod
    def GetToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.oauth.OauthService/GetToken',
            oauth_dot_oauth__pb2.TokenRequest.SerializeToString,
            common_dot_common__pb2.TokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SignToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.oauth.OauthService/SignToken',
            oauth_dot_oauth__pb2.SignTokenRequest.SerializeToString,
            common_dot_common__pb2.TokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetWhoAmI(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.oauth.OauthService/GetWhoAmI',
            oauth_dot_oauth__pb2.WhoAmIRequest.SerializeToString,
            oauth_dot_oauth__pb2.WhoAmIResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPublicKey(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/sensory.api.oauth.OauthService/GetPublicKey',
            oauth_dot_oauth__pb2.PublicKeyRequest.SerializeToString,
            oauth_dot_oauth__pb2.PublicKeyResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/event/event.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from sensory_cloud.generated.validate import validate_pb2 as validate_dot_validate__pb2
from sensory_cloud.generated.common import common_pb2 as common_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='v1/event/event.proto',
  package='sensory.api.v1.event',
  syntax='proto3',
  serialized_options=b'\n\034ai.sensorycloud.api.v1.eventB\026SensoryApiV1EventProtoP\001Z:gitlab.com/sensory-cloud/server/titan.git/pkg/api/v1/event',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14v1/event/event.proto\x12\x14sensory.api.v1.event\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x17validate/validate.proto\x1a\x13\x63ommon/common.proto\"M\n\x19PublishUsageEventsRequest\x12\x30\n\x06\x65vents\x18\x01 \x03(\x0b\x32 .sensory.api.v1.event.UsageEvent\"\xb1\x03\n\nUsageEvent\x12\x37\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xfa\x42\x05\xb2\x01\x02\x08\x01\x12\x19\n\x08\x64uration\x18\x02 \x01(\x03\x42\x07\xfa\x42\x04\"\x02(\x00\x12\x14\n\x02id\x18\x03 \x01(\tB\x08\xfa\x42\x05r\x03\xb0\x01\x01\x12\x1b\n\x08\x63lientId\x18\x04 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12:\n\x04type\x18\x05 \x01(\x0e\x32\".sensory.api.common.UsageEventTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x19\n\x05route\x18\x06 \x01(\tB\n\xfa\x42\x07r\x05\x10\x01\x18\xff\x03\x12\x38\n\x0ctechnologies\x18\x07 \x03(\x0e\x32\".sensory.api.common.TechnologyType\x12\x0e\n\x06models\x18\x08 \x03(\t\x12\x17\n\x0f\x61udioDurationMs\x18\t \x01(\x03\x12\x17\n\x0fvideoFrameCount\x18\n \x01(\x03\x12\x10\n\x08tenantId\x18\x0b \x01(\t\x12\x37\n\x10\x62illableFunction\x18\x0c \x01(\x0e\x32\x1d.sensory.api.common.ModelType\"\xc6\x03\n\x12UsageEventResponse\x12\x37\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x08\xfa\x42\x05\xb2\x01\x02\x08\x01\x12\x19\n\x08\x64uration\x18\x02 \x01(\x03\x42\x07\xfa\x42\x04\"\x02(\x00\x12\x14\n\x02id\x18\x03 \x01(\tB\x08\xfa\x42\x05r\x03\xb0\x01\x01\x12\x1b\n\x08\x63lientId\x18\x04 \x01(\tB\t\xfa\x42\x06r\x04\x10\x01\x18\x7f\x12:\n\x04type\x18\x05 \x01(\x0e\x32\".sensory.api.common.UsageEventTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x19\n\x05route\x18\x06 \x01(\tB\n\xfa\x42\x07r\x05\x10\x01\x18\xff\x03\x12\x38\n\x0ctechnologies\x18\x07 \x03(\x0e\x32\".sensory.api.common.TechnologyType\x12\x0e\n\x06models\x18\x08 \x03(\t\x12\x15\n\rbillableValue\x18\t \x01(\x03\x12\x15\n\rbillableUnits\x18\n \x01(\t\x12\x10\n\x08tenantId\x18\x0b \x01(\t\x12\x37\n\x10\x62illableFunction\x18\x0c \x01(\x0e\x32\x1d.sensory.api.common.ModelType\x12\x0f\n\x07\x63redits\x18\r \x01(\x01\"\xf5\x01\n\x15UsageEventListRequest\x12\x10\n\x08tenantId\x18\x01 \x01(\t\x12\x39\n\npagination\x18\x02 \x01(\x0b\x32%.sensory.api.common.PaginationOptions\x12)\n\x05\x61\x66ter\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12*\n\x06\x62\x65\x66ore\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x38\n\x11\x62illableFunctions\x18\x05 \x03(\x0e\x32\x1d.sensory.api.common.ModelType\"\x8e\x01\n\x16UsageEventListResponse\x12\x38\n\x06\x65vents\x18\x01 \x03(\x0b\x32(.sensory.api.v1.event.UsageEventResponse\x12:\n\npagination\x18\x02 \x01(\x0b\x32&.sensory.api.common.PaginationResponse\"\xbd\x01\n\x19GlobalEventSummaryRequest\x12\x0f\n\x07tenants\x18\x01 \x03(\t\x12)\n\x05\x61\x66ter\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12*\n\x06\x62\x65\x66ore\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x38\n\x11\x62illableFunctions\x18\x05 \x03(\x0e\x32\x1d.sensory.api.common.ModelType\"T\n\x11UsageEventSummary\x12?\n\tsummaries\x18\x01 \x03(\x0b\x32,.sensory.api.v1.event.UsageEventModelSummary\"\xa1\x01\n\x16UsageEventModelSummary\x12\x37\n\x10\x62illableFunction\x18\x01 \x01(\x0e\x32\x1d.sensory.api.common.ModelType\x12\r\n\x05units\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\x03\x12\r\n\x05\x63ount\x18\x04 \x01(\x03\x12\x0f\n\x07\x63redits\x18\x05 \x01(\x01\x12\x10\n\x08tenantId\x18\x06 \x01(\t\"\x1c\n\x1aPublishUsageEventsResponse2\xe0\x03\n\x0c\x45ventService\x12y\n\x12PublishUsageEvents\x12/.sensory.api.v1.event.PublishUsageEventsRequest\x1a\x30.sensory.api.v1.event.PublishUsageEventsResponse\"\x00\x12p\n\x11GetUsageEventList\x12+.sensory.api.v1.event.UsageEventListRequest\x1a,.sensory.api.v1.event.UsageEventListResponse\"\x00\x12n\n\x14GetUsageEventSummary\x12+.sensory.api.v1.event.UsageEventListRequest\x1a\'.sensory.api.v1.event.UsageEventSummary\"\x00\x12s\n\x15GetGlobalUsageSummary\x12/.sensory.api.v1.event.GlobalEventSummaryRequest\x1a\'.sensory.api.v1.event.UsageEventSummary\"\x00\x42t\n\x1c\x61i.sensorycloud.api.v1.eventB\x16SensoryApiV1EventProtoP\x01Z:gitlab.com/sensory-cloud/server/titan.git/pkg/api/v1/eventb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,validate_dot_validate__pb2.DESCRIPTOR,common_dot_common__pb2.DESCRIPTOR,])




_PUBLISHUSAGEEVENTSREQUEST = _descriptor.Descriptor(
  name='PublishUsageEventsRequest',
  full_name='sensory.api.v1.event.PublishUsageEventsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='events', full_name='sensory.api.v1.event.PublishUsageEventsRequest.events', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=125,
  serialized_end=202,
)


_USAGEEVENT = _descriptor.Descriptor(
  name='UsageEvent',
  full_name='sensory.api.v1.event.UsageEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='sensory.api.v1.event.UsageEvent.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005\262\001\002\010\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='duration', full_name='sensory.api.v1.event.UsageEvent.duration', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\004\"\002(\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='sensory.api.v1.event.UsageEvent.id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005r\003\260\001\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clientId', full_name='sensory.api.v1.event.UsageEvent.clientId', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='sensory.api.v1.event.UsageEvent.type', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005\202\001\002\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='route', full_name='sensory.api.v1.event.UsageEvent.route', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\007r\005\020\001\030\377\003', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='technologies', full_name='sensory.api.v1.event.UsageEvent.technologies', index=6,
      number=7, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='models', full_name='sensory.api.v1.event.UsageEvent.models', index=7,
      number=8, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='audioDurationMs', full_name='sensory.api.v1.event.UsageEvent.audioDurationMs', index=8,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='videoFrameCount', full_name='sensory.api.v1.event.UsageEvent.videoFrameCount', index=9,
      number=10, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tenantId', full_name='sensory.api.v1.event.UsageEvent.tenantId', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='billableFunction', full_name='sensory.api.v1.event.UsageEvent.billableFunction', index=11,
      number=12, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=205,
  serialized_end=638,
)


_USAGEEVENTRESPONSE = _descriptor.Descriptor(
  name='UsageEventResponse',
  full_name='sensory.api.v1.event.UsageEventResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='sensory.api.v1.event.UsageEventResponse.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005\262\001\002\010\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='duration', full_name='sensory.api.v1.event.UsageEventResponse.duration', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\004\"\002(\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='sensory.api.v1.event.UsageEventResponse.id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005r\003\260\001\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clientId', full_name='sensory.api.v1.event.UsageEventResponse.clientId', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\006r\004\020\001\030\177', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='sensory.api.v1.event.UsageEventResponse.type', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\005\202\001\002\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='route', full_name='sensory.api.v1.event.UsageEventResponse.route', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\372B\007r\005\020\001\030\377\003', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='technologies', full_name='sensory.api.v1.event.UsageEventResponse.technologies', index=6,
      number=7, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='models', full_name='sensory.api.v1.event.UsageEventResponse.models', index=7,
      number=8, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='billableValue', full_name='sensory.api.v1.event.UsageEventResponse.billableValue', index=8,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='billableUnits', full_name='sensory.api.v1.event.UsageEventResponse.billableUnits', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tenantId', full_name='sensory.api.v1.event.UsageEventResponse.tenantId', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='billableFunction', full_name='sensory.api.v1.event.UsageEventResponse.billableFunction', index=11,
      number=12, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='credits', full_name='sensory.api.v1.event.UsageEventResponse.credits', index=12,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=641,
  serialized_end=1095,
)


_USAGEEVENTLISTREQUEST = _descriptor.Descriptor(
  name='UsageEventListRequest',
  full_name='sensory.api.v1.event.UsageEventListRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tenantId', full_name='sensory.api.v1.event.UsageEventListRequest.tenantId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pagination', full_name='sensory.api.v1.event.UsageEventListRequest.pagination', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='after', full_name='sensory.api.v1.event.UsageEventListRequest.after', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='before', full_name='sensory.api.v1.event.UsageEventListRequest.before', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='billableFunctions', full_name='sensory.api.v1.event.UsageEventListRequest.billableFunctions', index=4,
      number=5, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1098,
  serialized_end=1343,
)


_USAGEEVENTLISTRESPONSE = _descriptor.Descriptor(
  name='UsageEventListResponse',
  full_name='sensory.api.v1.event.UsageEventListResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='events', full_name='sensory.api.v1.event.UsageEventListResponse.events', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pagination', full_name='sensory.api.v1.event.UsageEventListResponse.pagination', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1346,
  serialized_end=1488,
)


_GLOBALEVENTSUMMARYREQUEST = _descriptor.Descriptor(
  name='GlobalEventSummaryRequest',
  full_name='sensory.api.v1.event.GlobalEventSummaryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tenants', full_name='sensory.api.v1.event.GlobalEventSummaryRequest.tenants', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='after', full_name='sensory.api.v1.event.GlobalEventSummaryRequest.after', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='before', full_name='sensory.api.v1.event.GlobalEventSummaryRequest.before', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='billableFunctions', full_name='sensory.api.v1.event.GlobalEventSummaryRequest.billableFunctions', index=3,
      number=5, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1491,
  serialized_end=1680,
)


_USAGEEVENTSUMMARY = _descriptor.Descriptor(
  name='UsageEventSummary',
  full_name='sensory.api.v1.event.UsageEventSummary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='summaries', full_name='sensory.api.v1.event.UsageEventSummary.summaries', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1682,
  serialized_end=1766,
)


_USAGEEVENTMODELSUMMARY = _descriptor.Descriptor(
  name='UsageEventModelSummary',
  full_name='sensory.api.v1.event.UsageEventModelSummary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='billableFunction', full_name='sensory.api.v1.event.UsageEventModelSummary.billableFunction', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='units', full_name='sensory.api.v1.event.UsageEventModelSummary.units', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sensory.api.v1.event.UsageEventModelSummary.value', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='count', full_name='sensory.api.v1.event.UsageEventModelSummary.count', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='credits', full_name='sensory.api.v1.event.UsageEventModelSummary.credits', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tenantId', full_name='sensory.api.v1.event.UsageEventModelSummary.tenantId', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1769,
  serialized_end=1930,
)


_PUBLISHUSAGEEVENTSRESPONSE = _descriptor.Descriptor(
  name='PublishUsageEventsResponse',
  full_name='sensory.api.v1.event.PublishUsageEventsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1932,
  serialized_end=1960,
)

_PUBLISHUSAGEEVENTSREQUEST.fields_by_name['events'].message_type = _USAGEEVENT
_USAGEEVENT.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_USAGEEVENT.fields_by_name['type'].enum_type = common_dot_common__pb2._USAGEEVENTTYPE
_USAGEEVENT.fields_by_name['technologies'].enum_type = common_dot_common__pb2._TECHNOLOGYTYPE
_USAGEEVENT.fields_by_name['billableFunction'].enum_type = common_dot_common__pb2._MODELTYPE
_USAGEEVENTRESPONSE.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_USAGEEVENTRESPONSE.fields_by_name['type'].enum_type = common_dot_common__pb2._USAGEEVENTTYPE
_USAGEEVENTRESPONSE.fields_by_name['technologies'].enum_type = common_dot_common__pb2._TECHNOLOGYTYPE
_USAGEEVENTRESPONSE.fields_by_name['billableFunction'].enum_type = common_dot_common__pb2._MODELTYPE
_USAGEEVENTLISTREQUEST.fields_by_name['pagination'].message_type = common_dot_common__pb2._PAGINATIONOPTIONS
_USAGEEVENTLISTREQUEST.fields_by_name['after'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_USAGEEVENTLISTREQUEST.fields_by_name['before'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_USAGEEVENTLISTREQUEST.fields_by_name['billableFunctions'].enum_type = common_dot_common__pb2._MODELTYPE
_USAGEEVENTLISTRESPONSE.fields_by_name['events'].message_type = _USAGEEVENTRESPONSE
_USAGEEVENTLISTRESPONSE.fields_by_name['pagination'].message_type = common_dot_common__pb2._PAGINATIONRESPONSE
_GLOBALEVENTSUMMARYREQUEST.fields_by_name['after'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_GLOBALEVENTSUMMARYREQUEST.fields_by_name['before'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_GLOBALEVENTSUMMARYREQUEST.fields_by_name['billableFunctions'].enum_type = common_dot_common__pb2._MODELTYPE
_USAGEEVENTSUMMARY.fields_by_name['summaries'].message_type = _USAGEEVENTMODELSUMMARY
_USAGEEVENTMODELSUMMARY.fields_by_name['billableFunction'].enum_type = common_dot_common__pb2._MODELTYPE
DESCRIPTOR.message_types_by_name['PublishUsageEventsRequest'] = _PUBLISHUSAGEEVENTSREQUEST
DESCRIPTOR.message_types_by_name['UsageEvent'] = _USAGEEVENT
DESCRIPTOR.message_types_by_name['UsageEventResponse'] = _USAGEEVENTRESPONSE
DESCRIPTOR.message_types_by_name['UsageEventListRequest'] = _USAGEEVENTLISTREQUEST
DESCRIPTOR.message_types_by_name['UsageEventListResponse'] = _USAGEEVENTLISTRESPONSE
DESCRIPTOR.message_types_by_name['GlobalEventSummaryRequest'] = _GLOBALEVENTSUMMARYREQUEST
DESCRIPTOR.message_types_by_name['UsageEventSummary'] = _USAGEEVENTSUMMARY
DESCRIPTOR.message_types_by_name['UsageEventModelSummary'] = _USAGEEVENTMODELSUMMARY
DESCRIPTOR.message_types_by_name['PublishUsageEventsResponse'] = _PUBLISHUSAGEEVENTSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PublishUsageEventsRequest = _reflection.GeneratedProtocolMessageType('PublishUsageEventsRequest', (_message.Message,), {
  'DESCRIPTOR' : _PUBLISHUSAGEEVENTSREQUEST,
  '__module__' : 'v1.event.event_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.event.PublishUsageEventsRequest)
  })
_sym_db.RegisterMessage(PublishUsageEventsRequest)

UsageEvent = _reflection.GeneratedProtocolMessageType('UsageEvent', (_message.Message,), {
  'DESCRIPTOR' : _USAGEEVENT,
  '__module__' : 'v1.event.event_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.event.UsageEvent)
  })
_sym_db.RegisterMessage(UsageEvent)

UsageEventResponse = _reflection.GeneratedProtocolMessageType('UsageEventResponse', (_message.Message,), {
  'DESCRIPTOR' : _USAGEEVENTRESPONSE,
  '__module__' : 'v1.event.event_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.event.UsageEventResponse)
  })
_sym_db.RegisterMessage(UsageEventResponse)

UsageEventListRequest = _reflection.GeneratedProtocolMessageType('UsageEventListRequest', (_message.Message,), {
  'DESCRIPTOR' : _USAGEEVENTLISTREQUEST,
  '__module__' : 'v1.event.event_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.event.UsageEventListRequest)
  })
_sym_db.RegisterMessage(UsageEventListRequest)

UsageEventListResponse = _reflection.GeneratedProtocolMessageType('UsageEventListResponse', (_message.Message,), {
  'DESCRIPTOR' : _USAGEEVENTLISTRESPONSE,
  '__module__' : 'v1.event.event_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.event.UsageEventListResponse)
  })
_sym_db.RegisterMessage(UsageEventListResponse)

GlobalEventSummaryRequest = _reflection.GeneratedProtocolMessageType('GlobalEventSummaryRequest', (_message.Message,), {
  'DESCRIPTOR' : _GLOBALEVENTSUMMARYREQUEST,
  '__module__' : 'v1.event.event_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.event.GlobalEventSummaryRequest)
  })
_sym_db.RegisterMessage(GlobalEventSummaryRequest)

UsageEventSummary = _reflection.GeneratedProtocolMessageType('UsageEventSummary', (_message.Message,), {
  'DESCRIPTOR' : _USAGEEVENTSUMMARY,
  '__module__' : 'v1.event.event_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.event.UsageEventSummary)
  })
_sym_db.RegisterMessage(UsageEventSummary)

UsageEventModelSummary = _reflection.GeneratedProtocolMessageType('UsageEventModelSummary', (_message.Message,), {
  'DESCRIPTOR' : _USAGEEVENTMODELSUMMARY,
  '__module__' : 'v1.event.event_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.event.UsageEventModelSummary)
  })
_sym_db.RegisterMessage(UsageEventModelSummary)

PublishUsageEventsResponse = _reflection.GeneratedProtocolMessageType('PublishUsageEventsResponse', (_message.Message,), {
  'DESCRIPTOR' : _PUBLISHUSAGEEVENTSRESPONSE,
  '__module__' : 'v1.event.event_pb2'
  # @@protoc_insertion_point(class_scope:sensory.api.v1.event.PublishUsageEventsResponse)
  })
_sym_db.RegisterMessage(PublishUsageEventsResponse)


DESCRIPTOR._options = None
_USAGEEVENT.fields_by_name['timestamp']._options = None
_USAGEEVENT.fields_by_name['duration']._options = None
_USAGEEVENT.fields_by_name['id']._options = None
_USAGEEVENT.fields_by_name['clientId']._options = None
_USAGEEVENT.fields_by_name['type']._options = None
_USAGEEVENT.fields_by_name['route']._options = None
_USAGEEVENTRESPONSE.fields_by_name['timestamp']._options = None
_USAGEEVENTRESPONSE.fields_by_name['duration']._options = None
_USAGEEVENTRESPONSE.fields_by_name['id']._options = None
_USAGEEVENTRESPONSE.fields_by_name['clientId']._options = None
_USAGEEVENTRESPONSE.fields_by_name['type']._options = None
_USAGEEVENTRESPONSE.fields_by_name['route']._options = None

_EVENTSERVICE = _descriptor.ServiceDescriptor(
  name='EventService',
  full_name='sensory.api.v1.event.EventService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1963,
  serialized_end=2443,
  methods=[
  _descriptor.MethodDescriptor(
    name='PublishUsageEvents',
    full_name='sensory.api.v1.event.EventService.PublishUsageEvents',
    index=0,
    containing_service=None,
    input_type=_PUBLISHUSAGEEVENTSREQUEST,
    output_type=_PUBLISHUSAGEEVENTSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetUsageEventList',
    full_name='sensory.api.v1.event.EventService.GetUsageEventList',
    index=1,
    containing_service=None,
    input_type=_USAGEEVENTLISTREQUEST,
    output_type=_USAGEEVENTLISTRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetUsageEventSummary',
    full_name='sensory.api.v1.event.EventService.GetUsageEventSummary',
    index=2,
    containing_service=None,
    input_type=_USAGEEVENTLISTREQUEST,
    output_type=_USAGEEVENTSUMMARY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetGlobalUsageSummary',
    full_name='sensory.api.v1.event.EventService.GetGlobalUsageSummary',
    index=3,
    containing_service=None,
    input_type=_GLOBALEVENTSUMMARYREQUEST,
    output_type=_USAGEEVENTSUMMARY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_EVENTSERVICE)

DESCRIPTOR.services_by_name['EventService'] = _EVENTSERVICE

# @@protoc_insertion_point(module_scope)



import unittest
from unittest.mock import MagicMock

from sensory_cloud.config import Config, CloudHost
from sensory_cloud.services.health_service import HealthService
from sensory_cloud.generated.common.common_pb2 import (
    ServerHealthResponse,
    ServiceHealth,
)
from sensory_cloud.generated.health.health_pb2_grpc import HealthServiceStub


class MockHealthService(HealthService):
    def __init__(self, config: Config, health_client: HealthServiceStub):
        self._config = config
        self._health_client = health_client


class TestHealthService(unittest.TestCase):
    def test_get_health(self):
        cloud_host: CloudHost = CloudHost(host="domain.name")
        config: Config = Config(cloud_host=cloud_host, tenant_id="tenant-id")
        config.connect()

        response = ServerHealthResponse(
            isHealthy=True,
            serverVersion="1.2.3",
            services=[ServiceHealth(isHealthy=True, message="ok", name="test")],
        )

        health_client = HealthServiceStub(channel=config.channel)
        health_client.GetHealth = MagicMock(return_value=response)

        health_service: MockHealthService = MockHealthService(
            config=config, health_client=health_client
        )

        health_response: ServerHealthResponse = health_service.get_health()

        self.assertEqual(health_response, response)

        config.channel.close()


if __name__ == "__main__":
    unittest.main()
